@@ -1,7 +1,7 @@
 ## #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # <sure - utility belt for automated testing in python>
-# Copyright (C) <2010-2013>  Gabriel Falcão <gabriel@nacaolivre.org>
+# Copyright (C) <2010-2021>  Gabriel Falcão <gabriel@nacaolivre.org>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,12 +17,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 from __future__ import unicode_literals
 
+from six import text_type, PY2
+from six.moves import xrange
+
 import sure
 from sure.deprecated import that
+from sure.magic import is_cpython
 from sure import VariablesBag, expect
 from nose.tools import assert_equals, assert_raises
-from sure.six import text_type, text_type_name, compat_repr, PY3
-from sure.six.moves import xrange
+from sure.compat import compat_repr, safe_repr, text_type_name
 
 
 def test_setup_with_context():
@@ -55,7 +58,7 @@ def test_context_is_not_optional():
 
     assert that(it_crashes).raises(
         TypeError, (
-        "the function it_crashes defined at test_old_api.py line 53, is being "
+        "the function it_crashes defined at test_old_api.py line 56, is being "
         "decorated by either @that_with_context or @scenario, so it should "
         "take at least 1 parameter, which is the test context"),
     )
@@ -474,6 +477,28 @@ def test_that_looks_like():
     assert that('String\n with BREAKLINE').looks_like('string with breakline')
 
 
+def test_that_raises_does_raise_for_exception_type_mismatch():
+    "that(callable(RuntimeError)).raises(TypeError)"
+
+    error = r'.*should raise <class .*FooError.*, but raised <class .*BarError.*'
+
+    class FooError(Exception):
+        pass
+
+    class BarError(Exception):
+        pass
+
+    def my_function():
+        raise BarError('OOps')
+
+    try:
+        assert that(my_function).raises(FooError, 'OOps')
+        assert False, 'should never reach here'
+    except AssertionError as e:
+        import re
+        assert re.match(error, text_type(e))
+
+
 def test_that_raises_with_args():
     "that(callable, with_args=['foo']).raises(FooError)"
 
@@ -520,11 +545,13 @@ def test_that_contains_none():
 
     def assertions():
         # We can't use unicode in Py2, otherwise it will try to coerce
-        assert that('foobar' if PY3 else b'foobar').contains(None)
+        assert that(b'foobar' if PY2 else 'foobar').contains(None)
+
+    error_msg = "'in <string>' requires string as left operand, not NoneType" if is_cpython else "'NoneType' does not have the buffer interface"
 
     assert that(assertions).raises(
         TypeError,
-        u"'in <string>' requires string as left operand, not NoneType",
+        error_msg
     )
 
 
@@ -535,9 +562,10 @@ def test_that_none_contains_string():
         assert that(None).contains('bungalow')
         assert False, 'should not reach here'
     except Exception as e:
+        error_msg = "argument of type 'NoneType' is not iterable" if is_cpython else "'NoneType' object is not iterable"
         assert_equals(
             text_type(e),
-            u"argument of type 'NoneType' is not iterable",
+            error_msg
         )
 
 
@@ -577,8 +605,8 @@ def test_that_something_iterable_matches_another():
     "that(something_iterable).matches(another_iterable)"
 
     # types must be unicode in py3, bute bytestrings in py2
-    KlassOne = type('KlassOne' if PY3 else b'KlassOne', (object,), {})
-    KlassTwo = type('KlassTwo' if PY3 else b'KlassTwo', (object,), {})
+    KlassOne = type(b'KlassOne' if PY2 else 'KlassOne', (object,), {})
+    KlassTwo = type(b'KlassTwo' if PY2 else 'KlassTwo', (object,), {})
     one = [
         ("/1", KlassOne),
         ("/2", KlassTwo),
@@ -621,7 +649,7 @@ def test_within_fail():
     import time
     from sure import within, miliseconds
 
-    def sleepy():
+    def sleepy(*a):
         time.sleep(0.7)
 
     failed = False
@@ -727,7 +755,7 @@ def test_within_pass_utc():
 
 
 def test_that_is_a_matcher_should_absorb_callables_to_be_used_as_matcher():
-    u"that.is_a_matcher should absorb callables to be used as matcher"
+    "that.is_a_matcher should absorb callables to be used as matcher"
     @that.is_a_matcher
     def is_truthful(what):
         assert bool(what), '%s is so untrue' % (what)
@@ -889,7 +917,7 @@ def test_depends_on_failing_due_nothing_found():
     from sure import action_for, scenario
 
     fullpath = os.path.abspath(__file__).replace('.pyc', '.py')
-    error = 'the action "lonely_action" defined at %s:898 ' \
+    error = 'the action "lonely_action" defined at %s:926 ' \
         'depends on the attribute "something" to be available in the' \
         ' context. It turns out that there are no actions providing ' \
         'that. Please double-check the implementation' % fullpath
@@ -915,10 +943,10 @@ def test_depends_on_failing_due_not_calling_a_previous_action():
     from sure import action_for, scenario
 
     fullpath = os.path.abspath(__file__).replace('.pyc', '.py')
-    error = 'the action "my_action" defined at {0}:928 ' \
+    error = 'the action "my_action" defined at {0}:956 ' \
         'depends on the attribute "some_attr" to be available in the context.'\
         ' You need to call one of the following actions beforehand:\n' \
-        ' -> dependency_action at {0}:924'.replace('{0}', fullpath)
+        ' -> dependency_action at {0}:952'.replace('{0}', fullpath)
 
     def with_setup(context):
         @action_for(context, provides=['some_attr'])
@@ -1111,12 +1139,13 @@ def test_deep_equals_list_level1_fail_by_length_y_gt_x():
 
     assert that(assertions).raises(
         AssertionError, compat_repr(
-        "given\n" \
-        "X = ['one', 'yeah']\n" \
-        "    and\n" \
-        "Y = ['one', 'yeah', 'damn']\n" \
-        "Y has 3 items whereas X has only 2",
-    ))
+            "given\n"
+            "X = ['one', 'yeah']\n"
+            "    and\n"
+            "Y = ['one', 'yeah', 'damn']\n"
+            "Y has 3 items whereas X has only 2"
+        )
+    )
 
 
 def test_deep_equals_dict_level1_fails_missing_key_on_y():
@@ -1133,12 +1162,13 @@ def test_deep_equals_dict_level1_fails_missing_key_on_y():
 
     assert that(assertions).raises(
         AssertionError, compat_repr(
-        "given\n" \
-        "X = {'one': 'yeah'}\n" \
-        "    and\n" \
-        "Y = {'two': 'yeah'}\n" \
-        "X has the key 'one' whereas Y does not",
-    ))
+            "given\n"
+            "X = {{'one': 'yeah'}}\n"
+            "    and\n"
+            "Y = {{'two': 'yeah'}}\n"
+            "X has the key \"{0}\" whereas Y does not"
+        ).format(safe_repr('one'))
+    )
 
 
 def test_deep_equals_failing_basic_vs_complex():
@@ -1401,12 +1431,13 @@ def test_deep_equals_dict_level3_fails_missing_key():
 
     assert that(assertions).raises(
         AssertionError, compat_repr(
-        "given\n" \
-        "X = {'my::all_users': [{'age': 33, 'name': 'John'}]}\n" \
-        "    and\n" \
-        "Y = {'my::all_users': [{'age': 30, 'foo': 'bar', 'name': 'John'}]}\n" \
-        "X['my::all_users'][0] does not have the key 'foo' whereas Y['my::all_users'][0] has it",
-    ))
+            "given\n"
+            "X = {{'my::all_users': [{{'age': 33, 'name': 'John'}}]}}\n"
+            "    and\n"
+            "Y = {{'my::all_users': [{{'age': 30, 'foo': 'bar', 'name': 'John'}}]}}\n"
+            "X['my::all_users'][0] does not have the key \"{0}\" whereas Y['my::all_users'][0] has it"
+        ).format(safe_repr('foo'))
+    )
 
 
 def test_deep_equals_dict_level3_fails_extra_key():
@@ -1427,38 +1458,38 @@ def test_deep_equals_dict_level3_fails_extra_key():
 
     assert that(assertions).raises(
         AssertionError, compat_repr(
-        "given\n" \
-        "X = {'my::all_users': [{'age': 33, 'foo': 'bar', 'name': 'John'}]}\n" \
-        "    and\n" \
-        "Y = {'my::all_users': [{'age': 30, 'name': 'John'}]}\n" \
-        "X['my::all_users'][0] has the key 'foo' whereas Y['my::all_users'][0] does not",
-    ))
+        "given\n"
+        "X = {{'my::all_users': [{{'age': 33, 'foo': 'bar', 'name': 'John'}}]}}\n"
+        "    and\n"
+        "Y = {{'my::all_users': [{{'age': 30, 'name': 'John'}}]}}\n"
+        "X['my::all_users'][0] has the key \"{0}\" whereas Y['my::all_users'][0] does not"
+    ).format(safe_repr('foo')))
 
 
 def test_deep_equals_dict_level3_fails_different_key():
     "that() deep_equals(dict) failing on level 3 when has an extra key"
 
     something = {
-        u'my::all_users': [
-            {u'name': u'John', u'age': 33, u'foo': u'bar'},
+        'my::all_users': [
+            {'name': 'John', 'age': 33, 'foo': 'bar'},
         ],
     }
 
     def assertions():
         assert that(something).deep_equals({
-            u'my::all_users': [
-            {u'name': u'John', u'age': 33, u'bar': u'foo'},
+            'my::all_users': [
+            {'name': 'John', 'age': 33, 'bar': 'foo'},
             ],
         })
 
     assert that(assertions).raises(
         AssertionError, compat_repr(
-        "given\n" \
-        "X = {'my::all_users': [{'age': 33, 'foo': 'bar', 'name': 'John'}]}\n" \
-        "    and\n" \
-        "Y = {'my::all_users': [{'age': 33, 'bar': 'foo', 'name': 'John'}]}\n" \
-        "X['my::all_users'][0] has the key 'foo' whereas Y['my::all_users'][0] does not",
-    ))
+        "given\n"
+        "X = {{'my::all_users': [{{'age': 33, 'foo': 'bar', 'name': 'John'}}]}}\n"
+        "    and\n"
+        "Y = {{'my::all_users': [{{'age': 33, 'bar': 'foo', 'name': 'John'}}]}}\n"
+        "X['my::all_users'][0] has the key \"{0}\" whereas Y['my::all_users'][0] does not"
+    ).format(safe_repr('foo')))
 
 
 def test_deep_equals_list_level2_fail_by_length_x_gt_y():
@@ -1482,7 +1513,7 @@ def test_deep_equals_list_level2_fail_by_length_x_gt_y():
 def test_deep_equals_list_level2_fail_by_length_y_gt_x():
     "that(list) deep_equals(list) failing by length (len(Y) > len(X))"
 
-    something = [u'one', u'yeah']
+    something = ['one', 'yeah']
 
     def assertions():
         assert that(something).deep_equals(['one', 'yeah', 'damn'])
@@ -1513,10 +1544,10 @@ def test_function_decorated_with_wip_should_set_a_flag():
 def test_that_equals_fails():
     "that() equals(string) when it's supposed to fail"
 
-    something = u"else"
+    something = "else"
 
     def fail():
-        assert that(u'something').equals(something)
+        assert that('something').equals(something)
 
     assert that(fail).raises(
         AssertionError, compat_repr(
@@ -1549,9 +1580,9 @@ def test_deep_equals_weird():
     part1 = [
         ('Bootstraping Redis role', []),
         ('Restart scalarizr', []),
-        ('Rebundle server', [u'rebundle']),
-        ('Use new role', [u'rebundle']),
-        ('Restart scalarizr after bundling', [u'rebundle']),
+        ('Rebundle server', ['rebundle']),
+        ('Use new role', ['rebundle']),
+        ('Restart scalarizr after bundling', ['rebundle']),
         ('Bundling data', []),
         ('Modifying data', []),
         ('Reboot server', []),
@@ -1559,19 +1590,19 @@ def test_deep_equals_weird():
         ('Setup replication', []),
         ('Restart scalarizr in slave', []),
         ('Slave force termination', []),
-        ('Slave delete EBS', [u'ec2']),
-        ('Setup replication for EBS test', [u'ec2']),
+        ('Slave delete EBS', ['ec2']),
+        ('Setup replication for EBS test', ['ec2']),
         ('Writing on Master, reading on Slave', []),
         ('Slave -> Master promotion', []),
-        ('Restart farm', [u'restart_farm']),
+        ('Restart farm', ['restart_farm']),
     ]
 
     part2 = [
-        ('Bootstraping Redis role', [u'rebundle', u'rebundle', u'rebundle']),
+        ('Bootstraping Redis role', ['rebundle', 'rebundle', 'rebundle']),
         ('Restart scalarizr', []),
-        ('Rebundle server', [u'rebundle']),
-        ('Use new role', [u'rebundle']),
-        ('Restart scalarizr after bundling', [u'rebundle']),
+        ('Rebundle server', ['rebundle']),
+        ('Use new role', ['rebundle']),
+        ('Restart scalarizr after bundling', ['rebundle']),
         ('Bundling data', []),
         ('Modifying data', []),
         ('Reboot server', []),
@@ -1579,11 +1610,11 @@ def test_deep_equals_weird():
         ('Setup replication', []),
         ('Restart scalarizr in slave', []),
         ('Slave force termination', []),
-        ('Slave delete EBS', [u'ec2']),
-        ('Setup replication for EBS test', [u'ec2']),
+        ('Slave delete EBS', ['ec2']),
+        ('Setup replication for EBS test', ['ec2']),
         ('Writing on Master, reading on Slave', []),
         ('Slave -> Master promotion', []),
-        ('Restart farm', [u'restart_farm']),
+        ('Restart farm', ['restart_farm']),
     ]
 
     expect(that(part1).equals).when.called_with(part2).should.throw("")
